<?php
/**
 * For the full copyright and license information, please view the LICENSE file that was distributed with this source code.
 */
/**
 * This class holds the Multi Series {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts} XML specific functions
 *
 * @author Bart Vanderstukken <bart@digitalbase.eu>
 * @copyright (c) 2007-2008 Digital Base BVBA <{@link mailto:info@digitalbase.eu info@digitalbase.eu}> - See LICENSE
 * @package dbFusionChart
 *
 */
class dbFC_Multi extends dbFusionChartXML
{
  /**
   * Sets the attributes for the categories-node (creates it, if not yet created); for the x-axis data labels
   *
   * All arguments (most optional) correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param string $font valid font face
   * @param int $fontSize
   * @param string $fontColor hex code without #
   * @param array $atts extra attributes (name => value)
   */
  public function setCategoriesNodeAtts($font="", $fontSize=0, $fontColor="", $atts=array())
  {
    if($font) $atts["font"] = $font;
    if($fontSize) $atts["fontSize"] = $fontSize;
    if($fontColor) $atts["fontColor"] = $fontColor;

    self::assignAtts($this->getCategoriesNode(), $atts);
  }

  /**
   * Adds a set-node (no value) to each existing dataset (so # categories and # sets remain equal)
   *
   * Checks if category already exists, if so it just returns the existing category
   *
   * @param string $label appears on the x-axis of chart, used as identifier
   * @param array $atts name => value
   * @return DOMElement newly added, or already existing, category node
   * @see qAddCategory
   */
  public function addCategory($label, $atts=array())
  {
    if(($cat_node = $this->getCategory($label)) == null)
    {
      $atts[LABEL] = $label;

      $cat_node = $this->insertChild($this->getCategoriesNode(), "category", $atts);

      $datasets = $this->getDatasets();
      foreach ($datasets as $dataset)
      {
        $this->addSet($dataset);
      }
    }

    return $cat_node;
  }

  /**
   * Convenience method
   *
   * All arguments (most optional) correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param string $label appears on the x-axis of chart, used as identifier
   * @param boolean $showLabel
   * @param string $toolText
   * @param array $atts extra attributes (name => value)
   * @return DOMElement
   */
  public function qAddCategory($label, $showLabel=true, $toolText="", $atts=array())
  {
    if(!$showLabel) $atts[SHOWLABEL] = "0";
    if($toolText) $atts[TOOLTIP] = $toolText;

    return $this->addCategory($label, $atts);
  }

  /**
   * Quickly add categories (no attributes possible)
   *
   * @param array $categories <code>array("N. America", "Asia", "Europe", "Australia", "Africa")</code>
   */
  public function qAddCategories($categories)
  {
    foreach ($categories as $category)
    {
      $this->addCategory($category);
    }
  }

  /**
   * Quickly add categories with individual attributes
   *
   * @param array $categories <code>
   * array("N. America" => array(TOOLTIP => "North America"),
   *       "Asia" => array(),
   *       "Europe" => array(),
   *       "Australia" => array(),
   *       "Africa" => array())
   * </code>
   */
  public function qAddCategoriesWithAtts($categories)
  {
    foreach ($categories as $category => $atts)
    {
      $this->addCategory($category, $atts);
    }
  }

  /**
   * Adds as many sets (value=0) as there (currently) are categories to the new dataset (so # categories and # sets remain equal)
   *
   * @param array $atts name => value
   * @return DOMElement
   * @see qAddDataSet
   */
  public function addDataSet($atts=array())
  {
    $dataset = $this->insertRootChild("dataset", $atts);

    $max = $this->countCategories();
    for($i=0; $i < $max; $i++)
    {
      $this->addSet($dataset);
    }

    return $dataset;
  }

  /**
   * Convenience method
   *
   * All arguments are optional and correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param string $seriesName shown in legend, used as identifier
   * @param string $color hex color code without #
   *  - {@link http://www.fusioncharts.com/ FusionCharts V3}: for column chart, you can specify a list of comma separated hex codes to get a gradient plot
   * @param string $alpha transparency for entire dataset (0-100), can be overridden at <set> level
   *  - {@link http://www.fusioncharts.com/ FusionCharts V3}: you can also specify a comma separated list
   * @param boolean $showValues show/hide values of individual datasets
   * @param string $ratio ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *  - comma separated list of ratios
   *  - if you've opted to show columns as gradients, this attribute lets you control the ratio of each color
   *  - please see "{@link http://www.fusioncharts.com/Docs/Index.html Advanced Charting > Gradients}" for more information
   * @param boolean $dashed ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param boolean $includeLegend ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param array $atts
   * @return DOMElement
   */
  public function qAddDataSet($seriesName="", $color="", $alpha=null, $showValues=true, $ratio="", $dashed=false, $includeLegend=true, $atts=array())
  {
    if($seriesName) $atts["seriesName"] = $seriesName;
    if($color) $atts["color"] = $color;//"CCCCCC"; 
    if(isset($alpha)) $atts["alpha"] = "$alpha";
    if(!$showValues) $atts["showValues"] = "0";
    if(VERSION == "V3")
    {
      if($ratio) $atts["ratio"] = $ratio;
      if($dashed) $atts["dashed"] = "1";
      if(!$includeLegend) $atts["includeLegend"] = "0";
    }

    return $this->addDataSet($atts);
  }

  /**
   * Creates a dataset with 'set' children-nodes for $values
   *
   * Individual set attributes can be set via {@link setSetAtts}
   *
   * All optional arguments correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param array $values if #values != #categories => latest sets remain 0
   * @param string $seriesName shown in legend, used as identifier
   * @param string $color hex color code without #
   *  - {@link http://www.fusioncharts.com/ FusionCharts V3}: for column chart, you can specify a list of comma separated hex codes to get a gradient plot
   * @param string $alpha transparency for entire dataset (0-100), can be overridden at <set> level
   *  - {@link http://www.fusioncharts.com/ FusionCharts V3}: you can also specify a comma separated list
   * @param boolean $showValues show/hide values of individual datasets
   * @param string $ratio ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *  - comma separated list of ratios
   *  - if you've opted to show columns as gradients, this attribute lets you control the ratio of each color
   *  - please see "{@link http://www.fusioncharts.com/Docs/Index.html Advanced Charting > Gradients}" for more information
   * @param boolean $dashed ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param boolean $includeLegend ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param array $atts
   * @return DOMElement
   *
   * @throws Exception #values > #categories
   *
   * @todo catch index > #categories => add category "unknown" ? OR ignore ?
   */
  public function qAddDataSetWithValues($values, $seriesName="", $color="", $alpha=null, $showValues=true, $ratio="", $dashed=false, $includeLegend=true, $atts=array())
  {
    
    $dataset = $this->qaddDataSet($seriesName, $color, $alpha, $showValues, $ratio, $dashed, $includeLegend, $atts);

    $this->setData($dataset, $values);

    return $dataset;
  }

  /**
   * Adds an empty set-node to $dataset (value=0)
   *
   * USE {@link addDataSet}, IT'LL KEEP # OF CATEGORIES AND SETS IN A DATASET EQUAL
   *
   * @param DOMElement $dataset
   * @return DOMElement
   *
   */
  private function addSet(DOMElement $dataset)
  {
    return $this->insertChild($dataset, "set", array("value" => "0"));
  }

  /**
   * Sets values for all sets for $dataset
   * Non-associative arrays: in category order! #values <= (current) # categories
   * Associative arrays: label (cat) => value
   * If category doesn't exist, it'll be created, and while doing so all other datasets will get an extra empty set for that category
   *
   * @param DOMElement $dataset
   * @param array $values if #values != #categories => latest sets remain 0
   */
  public function setData(DOMElement $dataset, $values)
  {
    $i=0;
    foreach ($values as $key => $value)
    {
      if(is_string($key))
      {
        $associative = true;
        $this->qAddCategory($key); // make sure category exists
        $set = $this->setDataForLabel($dataset, $key, $value);
      }
      else
      {
        if(!$clear_to_go)
        {
          if($associative) { throw new Exception("Mixed usage: either you pass all data labeled or none"); }
          elseif(count($values) > $this->countCategories()) throw new Exception("The number of passed values (".count($values).") has to be less or equal to the (current) number of categories (".$this->countCategories().")");
          $clear_to_go = true;
        }

        $this->setDataOnPosition($dataset, $i++, $value);
      }
    }
  }

  /**
   * Sets $value for $index-th set-node, part of $dataset
   *
   * @param DOMElement $dataset
   * @param int $index valid index (< # categories)
   * @param string $value
   * @todo catch index > #categories => add category "unknown" ?
   */
  public function setDataOnPosition(DOMElement $dataset, $index, $value)
  {
    $this->getSet($dataset, $index)->setAttribute("value", $value);
  }

  /**
   * Sets $value for set-node, part of $dataset, related to category (identified by $label)
   *
   * @param DOMElement $dataset
   * @param string $label
   * @param string $value
   */
  public function setDataForLabel(DOMElement $dataset, $label, $value)
  {
    $this->getSetByLabel($dataset, $label)->setAttribute("value", $value);
  }

  /**
   * Sets $value for set-node, part of dataset (identified by $seriesName), related to category (identified by $label)
   *
   * Checks if category & dataset exist (else creates them)
   *
   * @param string $label
   * @param string $seriesName
   * @param string $value
   */
  public function qSetData($label, $seriesName, $value)
  {
    $this->getSetByLabel($this->qGetDataset($seriesName), $label)->setAttribute("value", $value);
  }

  /**
   * Set attributes for individual data items
   *
   * All optional arguments correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param DOMElement $set set-node, can be retrieved by getSet or getSetByLabel
   * @param string $color hex color code without #; overrides the dataset level value
   * @param URL $link define links for individual data items
   *  - {@link http://www.fusioncharts.com/ FusionCharts V3}: please see "{@link http://www.fusioncharts.com/Docs/Index.html Advanced Charting > Drill Down Charts}" for more information
   *  - {@link http://www.fusioncharts.com/free FusionCharts FREE}: to open a link in a new window, just put n- in front of the link
   * @param int $alpha transparency (0-100); overrides the data-set level value
   * @param string $toolText by default, FusionCharts shows the series name, category name and value as tool tip text, but if you want to display more information, you can use this attribute to specify the same <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param boolean $showValue show/hide values of individual data items <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param boolean $dashed border of this data item should appear as dashed? <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param array $atts extra attributes (name => value)
   */
  public function setSetAtts(DOMElement $set, $color="", $link="", $toolText="", $showValue=true, $dashed=false, $alpha=null, $atts=array())
  {
    if($color) $atts["color"] = $color;
    if($link) $atts["link"] = urlencode($link);
    if(isset($alpha)) $atts["alpha"] = "$alpha";
    if(VERSION == "V3")
    {
      if($toolText) $atts["toolText"] = $toolText;
      if(!$showValue) $atts["showValue"] = "0";
      if($dashed) $atts["dashed"] = "1";
    }

    self::assignAtts($set, $atts);
  }

  /**
   * @param string $before_label label of category before which the vertical data seperator line should be inserted
   * @param array $atts name => value
   */
  protected function addVline($before_label, $atts=array())
  {
    $this->insertChild($this->getCategoriesNode(), "vLine", $atts, $this->getCategory($before_label));
  }

  /**
   * Creates the categories node, if not yet created
   *
   * @return DOMElement
   */
  public function getCategoriesNode()
  {
    return $this->getUniqueNode("categories");
  }

  /**
   * @return DOMNodeList all category-nodes
   */
  public function getCategories()
  {
    return $this->getCategoriesNode()->getElementsByTagName("category");
  }

  /**
   * @param string $label
   * @return DOMElement category-node identified by $label
   */
  public function getCategory($label)
  {
    $cats = $this->getCategories();
    /* @var $category_node DOMElement */
    foreach ($cats as $category_node)
    {
      if($category_node->getAttribute(LABEL) == $label) return $category_node;
    }
  }

  /**
   * @return int number of category-nodes
   */
  public function countCategories()
  {
    return $this->getCategories()->length;
  }

  /**
   * @return DOMNodeList all dataset-nodes
   */
  public function getDatasets()
  {
    return $this->getChartNode()->getElementsByTagName("dataset");
  }

  /**
   * @param string $seriesName
   * @return DOMElement dataset-node identified by $seriesName
   */
  public function getDataset($seriesName)
  {
    $datasets = $this->getDatasets();
    /* @var $dataset DOMElement */
    foreach ($datasets as $dataset)
    {
      if($dataset->getAttribute("seriesName") == $seriesName) return $dataset;
    }
  }

  /**
   * Creates the dataset if it didn't yet existed
   *
   * @param string $seriesName
   * @return DOMElement dataset-node identified by $seriesName
   */
  public function qGetDataset($seriesName)
  {
    if(($dataset = $this->getDataset($seriesName)) == null) $dataset = $this->qAddDataSet($seriesName);

    return $dataset;
  }

  /**
   * @return DOMNodeList
   */
  public function getSets(DOMElement $dataset)
  {
    return $dataset->getElementsByTagName("set");
  }

  /**
   * @param DOMElement $dataset
   * @param int $pos
   * @return DOMElement set-node, part of $dataset, on passed $position
   */
  public function getSet(DOMElement $dataset, $pos)
  {
    return $this->getSets($dataset)->item($pos);
  }

  /**
   * @param DOMElement $dataset
   * @param string $label
   * @return DOMElement set-node, part of $dataset, related to category (identified by $label)
   */
  public function getSetByLabel(DOMElement $dataset, $label)
  {
    return $this->getSet($dataset, $this->getCategoryIndex($label));
  }

  /**
   * @param string $label
   * @return int position of category-node
   */
  public function getCategoryIndex($label)
  {
    $cats = $this->getCategories();

    $i = 0;
    foreach ($cats as $category_node)
    {
      if($category_node->getAttribute(LABEL) == $label) return $i;
      $i++;
    }
  }
}