<?php
/**
 * For the full copyright and license information, please view the LICENSE file that was distributed with this source code.
 */

/**
 *
 */
include_once("config.php");

/**
 * This class is used to create the required XML document
 *
 * @author Bart Vanderstukken <bart@digitalbase.eu>
 * @copyright (c) 2007-2008 Digital Base BVBA <{@link mailto:info@digitalbase.eu info@digitalbase.eu}> - See LICENSE
 * @package dbFusionChart
 * @see http://www.php.net/dom
 * @todo create separate objects for different versions (Free & V3)
 */
abstract class dbFusionChartXML extends DOMDocument
{
  private static $colors = array("#839f37", "#c04a86", "#378899", "#9a5d38", "#4b525a", );
  private $used_colors;
  private $color_counter = 0;

  /**
   * @param array $chartatts
   */
  public function __construct($chartatts=array())
  {
    parent::__construct('1.0', 'iso-8859-1');
    $this->resetColors();
    $this->formatOutput = true;
    $this->insertChild($this, ROOTTAG, $chartatts);
    $this->init();
  }

  /**
   * Empty method, called at construction
   * Override this method in a descendent to "save" the way a chart's data is being constructed, so it can be used again
   */
  public function init() { }

  /**
   * Sets corresponding attributes to the root node
   *
   * All arguments are optional and correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param string $caption appears at the top of the chart
   * @param string $xAxisName x-Axis text title (if the chart supports axis)
   * @param string $yAxisName y-Axis text title (if the chart supports axis)
   * @param string $subcaption
   * @param array $atts extra attributes (name => value)
   * @return dbFusionChartXML
   */
  public function setChartAtts($caption="", $xAxisName="", $yAxisName="", $subcaption="", $atts=array())
  {
    if($caption) $atts["caption"] = $caption;
    if($xAxisName) $atts["xaxisname"] = $xAxisName;
    if($yAxisName) $atts["yaxisname"] = $yAxisName;
    if($subcaption) $atts["subCaption"] = $subcaption;

    self::assignAtts($this->getChartNode(), $atts);
  }

  /**
   * Convenience method <br />
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * All arguments (most optional) correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param string $before_label label of element (set/category) before which the vertical data seperator line should be inserted
   * @param string $color hex code without #
   * @param int $thickness in pixels
   * @param int $alpha of the vertical separator line
   * @param boolean $dashed
   * @param int $dashLen in pixels
   * @param int $dashGap in pixels
   * @param array $atts extra attributes (name => value)
   */
  public function qAddVline($before_label, $color, $thickness=1, $alpha=0, $dashed=false, $dashLen=0, $dashGap=0, $atts=array())
  {
    if(VERSION == "V3")
    {
      $atts["color"] = $color;
      $atts["thickness"] = $thickness;
      if($alpha) $atts["alpha"] = $alpha;
      if($dashed) $atts["dashed"] = "1";
      if($dashLen) $atts["dashLen"] = $dashLen;
      if($dashGap) $atts["dashGap"] = $dashGap;

      $this->addVline($before_label, $atts);
    }
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  /**
   * Inserts vertical separator line that help you separate blocks of data. These lines run through the height of the chart, thereby segregating data into different blocks. In case of bar charts, they are horizontal and run through the width of chart.
   *
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @param string $before_label label of element (set/category) before which the vertical data seperator line should be inserted
   * @param array $atts name => value
   * @see qAddVline
   */
  abstract protected function addVline($before_label, $atts=array());

  /**
   * @param array $atts name => value
   * @see qAddTrendline
   */
  public function addTrendLine($atts=array())
  {
    $this->insertChild($this->getTrendlinesNode(), "line", $atts);
  }

  /**
   * Convenience method
   *
   * All arguments are optional and correspond with the attributes as described in the {@link http://www.fusioncharts.com/free/docs/?gMenuItemId=19 FusionCharts Documentation} to date.<br />
   * Attributes that aren't included in the argument list, because we didn't regard them as current or because of changes/updates in the underlying FusionCharts software, can be passed in an array (att_name => value) via $atts.
   *
   * Default values are ignored; if the default value seems to be wrong, change it in the source code and please {@link mailto:info@digitalbase.eu inform us!}
   *
   * @param int $startValue starting y-axis value for the trendline
   * @param int $endValue ending y-axis value for the trendline; if not provided, assumed to be same value as startValue
   * @param string $displayValue caption for the trend line by its side; if not provided = startValue
   * @param string $color color of the trend line and its associated text; hex code without #
   * @param boolean $isTrendZone line, or a zone (filled colored rectangle)
   * @param boolean $showOnTop over data plots or under them?
   * @param int $thickness if you've opted to show the trend as a line, in pixels
   * @param int $alpha transparency (0-100)
   * @param boolean $dashed if you've opted to show the trend as a line <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param int $dashLen in pixels <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param int $dashGap in pixels <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param boolean $valuesOnRight <br />
   *   ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   * @param array $atts extra attributes (name => value)
   */
  public function qAddTrendline($startValue=null, $endValue=null, $displayValue="", $color="", $isTrendZone=false, $showOnTop=true, $thickness=0, $alpha=null, $dashed=false, $dashLen=0, $dashGap=0, $valuesOnRight=false, $atts=array())
  {
    if(isset($startValue)) $atts["startValue"] = $startValue;
    if(isset($endValue)) $atts["endValue"] = $endValue;
    if($displayValue) $atts["displayValue"] = $displayValue;
    if($color) $atts["color"] = $color;
    if($isTrendZone) $atts["isTrendZone"] = "1";
    if(!$showOnTop) $atts["showOnTop"] = "0";
    if($thickness) $atts["thickness"] = $thickness;
    if(isset($alpha)) $atts["alpha"] = $alpha;
    if(VERSION == "V3")
    {
      if($dashed) $atts["dashed"] = "1";
      if($dashLen) $atts["dashLen"] = $dashLen;
      if($dashGap) $atts["dashGap"] = $dashGap;
      if($valuesOnRight) $atts["valuesOnRight"] = "1";
    }

    $this->addTrendLine($atts);
  }

  /**
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @param array $atts name => value
   * @see qAddStyle
   * @todo add style support
   */
  public function addStyle($atts)
  {
    if(VERSION == "V3")
    {
      $this->insertChild($this->getDefinitionNode(), "style", $atts);
      $this->insertChild($this->getApplicationNode(), "apply", $atts);
    }
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  /**
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @todo add style support
   */
  public function qAddStyle(/*$startValue, $color, $displayValue*/)
  {
    if(VERSION == "V3")
    {
//    $this->addStyle(array("startValue" => $startValue, "color" => $color, "displayvalue" => $displayValue));
    }
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  /**
   * Method to assign multiple attributes to a {@link http://www.php.net/dom DOMElement} at once
   *
   * @param DOMElement $el
   * @param array $atts name => value
   */
  protected static function assignAtts($el, $atts)
  {
    foreach ($atts as $name => $value)
    {
      $el->setAttribute($name, $value);
    }
  }

  /**
   * Convenience method
   *
   * Calls {@link insertChild} with {@link getChartNode root node} as parent
   *
   * @param string $name new (child) node's name
   * @param array $atts attributes for child node (name => value)
   * @param DOMNode $refnode if not supplied, new node is appended to the children
   * @return DOMElement created child node
   */
  public function insertRootChild($name, $atts=array(), $refnode=null)
  {
    return $this->insertChild($this->getChartNode(), $name, $atts, $refnode);
  }

  /**
   * Creates a new "$name"-node, assigns $atts and inserts it as child for $parent before $refnode
   *
   * @param DOMNode $parent
   * @param string $name new (child) node's name
   * @param array $atts attributes for child node (name => value)
   * @param DOMNode $refnode if not supplied, new node is appended to the children
   * @return DOMElement created child node
   */
  public function insertChild(DOMNode $parent, $name, $atts=array(), $refnode=null)
  {
    $el = $this->createElement($name);
    self::assignAtts($el, $atts);
    $parent->insertBefore($el, $refnode);

    return $el;
  }

  /**
   * @see ROOTTAG
   * @return DOMElement
   */
  public function getChartNode()
  {
    $nodelist = $this->getElementsByTagName(ROOTTAG);
    return $nodelist->item(0);
  }

  /**
   * Creates "$name"-node, if not yet created
   *
   * @param string $name
   * @return DOMElement
   */
  protected function getUniqueNode($name)
  {
    $nodelist = $this->getChartNode()->getElementsByTagName($name);

    if($nodelist->length == 0)
    {
      $node = $this->insertRootChild($name);
    }
    else $node = $nodelist->item(0);

    return $node;
  }

  /**
   * Creates the trendlines-node, if not yet created
   *
   * @return DOMElement
   */
  public function getTrendlinesNode()
  {
    return $this->getUniqueNode("trendlines");
  }

  /**
   * Creates the styles-node, if not yet created
   *
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @return DOMElement
   */
  public function getStylesNode()
  {
    if(VERSION == "V3") return $this->getUniqueNode("styles");
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  /**
   * Creates the definition-node (child of styles-node), if not yet created
   *
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @return DOMElement
   */
  public function getDefinitionNode()
  {
    if(VERSION == "V3") return $this->getUniqueNode("definition");
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  /**
   * Creates the application-node (child of styles-node), if not yet created
   *
   * ONLY AVAILABLE IN {@link http://www.fusioncharts.com/ NON-FREE VERSION} OF FUSIONCHARTS
   *
   * @return DOMElement
   */
  public function getApplicationNode()
  {
    if(VERSION == "V3") return $this->getUniqueNode("application");
    else throw new Exception("Only available in non-free version. If you have purchased FusionCharts V3, change the VERSION constant in config.php");
  }

  public function randomColor($repeat=false)
  {
    if($this->color_counter < count(self::$colors))
    {
      $color = self::$colors[$this->color_counter++];
    }
    else $color = self::trulyRandomColor();

    if(!$repeat)
    {
      while (in_array($color, $this->used_colors))
      {
        $color = $this->randomColor(false);
      }

      $this->used_colors[] = $color;
    }

    return $color;
  }

  private static function trulyRandomColor()
  {
    return self::randomColorRGB().self::randomColorRGB().self::randomColorRGB();
  }

  private static function randomColorRGB()
  {
    return dbTimeBlock::leading_zeros(dechex(rand(0,255)),2);
  }

  public function resetColors()
  {
    $this->used_colors = array();
  }
}