<?php

/*
 * This file is part of the sfPropelFinder package.
 * 
 * (c) 2007 François Zaninotto <francois.zaninotto@symfony-project.com>
 * 
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
 
class sfPropelFinder
{
  protected $peerClass = null;
  protected $databaseMap = null;
  protected $criteria = null;
  protected $relations = null;
  protected $latestQuery = '';
  
  public function __construct($class = '')
  {
    $this->criteria = new Criteria();
    if($class)
    {
      $this->setPeerClass($class.'Peer');
    }
    if($this->peerClass)
    {
      $this->initialize();
    }
  }
  
  public function initialize()
  {
    $mapBuilder = call_user_func(array($this->peerClass, 'getMapBuilder'));
    $mapBuilder->doBuild();
    $this->databaseMap = $mapBuilder->getDatabaseMap();
  }
  
  public function getPeerClass()
  {
    return $this->peerClass;
  }

  public function setPeerClass($peerClass)
  {
    $this->relations[]= $peerClass;
    $this->peerClass = $peerClass;
    return $this;
  }
  
  public function getCriteria()
  {
    return $this->criteria;
  }
  
  public function setCriteria($criteria)
  {
    $this->criteria = $criteria;
    return $this;
  }

  public function reinitCriteria()
  {
    $this->criteria = new Criteria();
    return $this;
  }
  
  public function getLatestQuery()
  {
    return $this->latestQuery;
  }
  
  public function updateLatestQuery()
  {
    $this->latestQuery = Propel::getConnection()->getLastExecutedQuery();
  }
  
  // Finder Initializers
  
  public static function from($class)
  {
    $me = __CLASS__;
    $finder = new $me($class);
    
    return $finder;
  }
  
  // Finder Executers
  
  public function count($con = null, $reinitCriteria = true)
  {
    $ret = call_user_func(array($this->getPeerClass(), 'doCount'), $this->criteria, $con);
    $this->updateLatestQuery();
    if($reinitCriteria)
    {
      $this->reinitCriteria();
    }
    
    return $ret;
  }
  
  public function find($limit = null, $con = null, $reinitCriteria = false)
  {
    if($limit)
    {
      $this->criteria->setLimit($limit);
    }
    $ret = call_user_func(array($this->getPeerClass(), 'doSelect'), $this->criteria, $con);
    $this->updateLatestQuery();
    if($reinitCriteria)
    {
      $this->reinitCriteria();
    }
    
    return $ret;
  }
  
  public function findOne($con = null, $reinitCriteria = true)
  {
    $ret = call_user_func(array($this->getPeerClass(), 'doSelectOne'), $this->criteria, $con);
    $this->updateLatestQuery();
    if($reinitCriteria)
    {
      $this->reinitCriteria();
    }
    
    return $ret;
  }
  
  public function findPk($pk, $con = null)
  {
    if(is_array($pk))
    {
      $ret = call_user_func(array($this->getPeerClass(), 'retrieveByPks'), $pk, $con);
    }
    else
    {
      $ret = call_user_func(array($this->getPeerClass(), 'retrieveByPk'), $pk, $con);
    }
    $this->updateLatestQuery();
    
    return $ret;
  }
  
  public function delete($con = null, $reinitCriteria = true)
  {
    $deleteCriteria = clone $this->criteria;
    if($deleteCriteria->equals(new Criteria()))
    {
      // delete will delete nothing when passed an empty criteria
      // while it should, in fact, delete all
      $fieldNames = call_user_func(array($this->getPeerClass(), 'getFieldNames'), BasePeer::TYPE_COLNAME);
      $firstFieldName = $fieldNames[0];
      $deleteCriteria->add($firstFieldName, true, Criteria::BINARY_OR);
    }
    $ret = call_user_func(array($this->getPeerClass(), 'doDelete'), $deleteCriteria, $con);
    $this->updateLatestQuery();
    if($reinitCriteria)
    {
      $this->reinitCriteria();
    }
    
    return $ret;
  }
  
  // Finder Filters
  
  /**
   * Finder Fluid Interface for Criteria::setDistinct()
   */
  public function distinct()
  {
    $this->criteria->setDistinct();
    
    return $this;
  }
  
  /**
   * Finder Fluid Interface for Criteria::add()
   *
   * @param      string $critOrColumn The column to run the comparison on, or Criterion object.
   * @param      mixed  $value The value to be matched
   * @param      string $comparison Comparison operator (leave empty for equal)
   *
   * @return     sfPropelFinder the current finder object
   */
  public function where($critOrColumn, $value = null, $comparison = null)
  {
    $this->criteria->add($critOrColumn, $value, $comparison);
    
    return $this;
  }
  
  /**
   * Magic version of where()
   * Infers $column, $value, $comparison from $columnName and some optional arguments
   * Examples:
   *   $articleFinder->_where('IsPublished')
   *    => $c->add(ArticlePeer::IS_PUBLISHED, true)
   *   $articleFinder->_where('CommentId', 3)
   *    => $c->add(ArticlePeer::COMMENT_ID, 3)
   *   $articleFinder->_where('Title', 'like', '%foo')
   *    => $c->add(ArticlePeer::TITLE, '%foo', Criteria::LIKE)
   *
   * @param      string  $columnName PHPName of the column bearing the condition
   * @param      array   $arguments  Optional array of arguments
   *
   * @return     sfPropelFinder the current finder object
   */
  public function _where($columnName, $arguments = array())
  {
    $column = $this->getColName($columnName);
    $comparison = Criteria::EQUAL;
    switch (count($arguments))
    {
      case 0:
        $value = true;
        break;
      case 1:
        $value = array_shift($arguments);
        break;
      case 2:
        $comparison = array_shift($arguments);
        $comparisonUp = trim(strtoupper($comparison));
        if(in_array($comparisonUp, array('LIKE', 'NOT LIKE', 'ILIKE', 'NOT ILIKE', 'IN', 'NOT IN', 'IS NULL', 'IS NOT NULL')))
        {
          $comparison = ' '.$comparisonUp.' ';
        }
        $value = array_shift($arguments);
        break;
      default:
        throw new Exception('{sfPropelFinder} whereXXX can only be called with one or two arguments');
    }
    
    return $this->where($column, $value, $comparison);
  }

  /**
   * Finder Fluid Interface for Criteria::addAscendingOrderByColumn()
   * and Criteria::addDescendingOrderByColumn()
   */
  public function orderBy($column, $order = Criteria::ASC)
  {
    switch ($order)
    {
      case Criteria::ASC:
        $this->criteria->addAscendingOrderByColumn($column);
        break;
      case Criteria::DESC:
        $this->criteria->addDescendingOrderByColumn($column);
        break;
      default:
        throw new Exception('{sfPropelFinder} orderBy only accepts "asc" or "desc" as argument');
    }
    
    return $this;
  }
  
  /**
   * Magic version of orderBy()
   * Infers $column and $order from $columnName and some optional arguments
   * Examples:
   *   $articleFinder->_orderBy('CreatedAt')
   *    => $c->addAscendingOrderByColumn(ArticlePeer::CREATED_AT)
   *   $articlefinder->_orderBy('CategoryId', 'desc')
   *    => $c->addDescendingOrderByColumn(ArticlePeer::CATEGORY_ID)
   */
  public function _orderBy($columnName, $arguments = array())
  {
    $column = $this->getColName($columnName);
    $order = strtoupper(array_shift($arguments));
    if(!$order)
    {
      $order = Criteria::ASC;
    }
    
    return $this->orderBy($column, $order);
  }
  
  /**
   * Finder Fluid Interface for Criteria::addJoin()
   */
  public function join($column1, $column2, $operator = null)
  {
    $this->criteria->addJoin($column1, $column2, $operator);
    
    return $this;
  }
  
  /**
   * Magic version of join()
   * Infers $column1, $column2 and $operator from $relatedClass and some optional arguments
   * Uses the Propel column maps, based on the schema, to guess the related columns
   * Examples:
   *   $articleFinder->_join('Comment')
   *    => $c->addJoin(ArticlePeer::ID, CommentPeer::ARTICLE_ID)
   *   $articleFinder->_join('Category', 'RIGHT JOIN')
   *    => $c->addJoin(ArticlePeer::CATEGORY_ID, CategoryPeer::ID, Criteria::RIGHT_JOIN)
   */
  public function _join($relatedClass, $arguments = array())
  {
    list($column1, $column2) = $this->getRelation($relatedClass);
    $this->relations[]= $relatedClass.'Peer';
    $operator = array_shift($arguments);
    if(!$operator)
    {
      $operator = null;
    }
    
    return $this->join($column1, $column2, $operator);
  }

  public function getRelation($phpName)
  {
    foreach($this->relations as $peerClass)
    {
      // try to find many to one or one to one relationship
      if($relation = $this->findRelation($phpName, $peerClass))
      {
        return $relation;
      }
      // try to find one to many relationship
      if($relation = $this->findRelation(str_replace('Peer', '', $peerClass), $phpName.'Peer'))
      {
        return $relation;
      }
    }
    throw new Exception(sprintf('{sfPropelFinder} %s has no %s related table', $this->peerClass, $phpName));
  }
  
  protected function findRelation($phpName, $peerClass)
  {
    foreach ($this->getColumnsForPeerClass($peerClass) as $c)
    {
      if ($c->isForeignKey())
      {
        if(sfInflector::camelize($c->getRelatedTableName()) == $phpName)
        {
          return array(
            constant($peerClass.'::'.$c->getColumnName()),
            $c->getRelatedName()
          );
        }
      }
    }
    
    return false;
  }
  
  protected function getColumnsForPeerClass($peerClass)
  {
    if(class_exists($peerClass))
    {
      $tableMap = call_user_func(array($peerClass, 'getTableMap'));
      return $tableMap->getColumns();
    }
    return false;
  }
  
  protected function getColName($phpName, $peerClass = null)
  {
    if(strpos($phpName, '_') !== false)
    {
      list($class, $phpName) = split('_', $phpName);
      $peerClass = $class.'Peer';
    }
    if(!$peerClass)
    {
      $peerClass = $this->peerClass;
    }
    try
    {
      $column = call_user_func(array($peerClass, 'translateFieldName'), $phpName, BasePeer::TYPE_PHPNAME, BasePeer::TYPE_COLNAME);
      return $column;
    }
    catch (PropelException $e)
    {
      throw new Exception(sprintf('{sfPropelFinder} %s has no %s column', $peerClass, $phpName));
    }
  }
  
  public function __call($name, $arguments)
  {
    if(strpos($name, 'where') === 0)
    {
      return $this->_where(substr($name, 5), $arguments);
    }
    if(strpos($name, 'orderBy') === 0)
    {
      return $this->_orderBy(substr($name, 7), $arguments);
    }
    if(strpos($name, 'join') === 0)
    {
      return $this->_join(substr($name, 4), $arguments);
    }
    if(method_exists($this->criteria, $name))
    {
      call_user_func_array(array($this->criteria, $name), $arguments);
      return $this;
    }
    throw new Exception(sprintf('{sfPropelFinder} Undefined method %s', $name));
  }
}